@@ -18,46 +18,51 @@
 
 ## mapping helper
 
-map <- function(x,
-  range = c(0, 1),
-  from.range = NA) {
-  if (is.null(from.range) || any(is.na(from.range)))
+map <- function(
+    x,
+    range = c(0, 1),
+    from.range = NA) {
+  if (is.null(from.range) || any(is.na(from.range))) {
     from.range <- range(x, na.rm = TRUE)
-  
+  }
+
   ## check if all values are the same
   if (!diff(from.range)) {
-    if (is.matrix(x))
+    if (is.matrix(x)) {
       return(matrix(
         mean(range),
         ncol = ncol(x),
         nrow = nrow(x),
         dimnames = dimnames(x)
       ))
-    else
+    } else {
       return(structure(rep(mean(range), length(x)), names = names(x)))
-    
+    }
   }
-  
+
   ## map to [0,1]
   x <- (x - from.range[1])
   x <- x / diff(from.range)
   ## handle single values
-  if (diff(from.range) == 0)
+  if (diff(from.range) == 0) {
     x <- 0
-  
+  }
+
   ## map from [0,1] to [range]
-  if (range[1] > range[2])
+  if (range[1] > range[2]) {
     x <- 1 - x
+  }
   x <- x * (abs(diff(range))) + min(range)
-  
+
   x[x < min(range) | x > max(range)] <- NA
-  
+
   x
 }
 
 
-map_int <- function(x,
-  range = c(1, 100),
-  from.range = NA) {
+map_int <- function(
+    x,
+    range = c(1, 100),
+    from.range = NA) {
   floor(map(x, c(range[1], range[2] + .9), from.range))
 }
